/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

/**
 * @file
 * Page title.
 */

.page-title {
  /*
    Necessary to properly display the star icon from core's shortcut
    module.
  */
  display: inline;
}
